@@ -11,8 +11,10 @@ use File::Path ();
 BEGIN {
     # Environment variables which might effect our testing
     my @delete_env_keys = qw(
+        HOME
         DEVEL_COVER_OPTIONS
         MODULEBUILDRC
+        PERL_MB_OPT
         HARNESS_TIMER
         HARNESS_OPTIONS
         HARNESS_VERBOSE
@@ -50,24 +52,12 @@ BEGIN {
   # In case the test wants to use our other bundled
   # modules, make sure they can be loaded.
   my $t_lib = File::Spec->catdir('t', 'bundled');
+  push @INC, $t_lib; # Let user's installed version override
 
-  unless ($ENV{PERL_CORE}) {
-    push @INC, $t_lib; # Let user's installed version override
-  } else {
-    # We change directories, so expand @INC and $^X to absolute paths
-    # Also add .
-    @INC = (map(File::Spec->rel2abs($_), @INC), ".");
-    $^X = File::Spec->rel2abs($^X);
-
-    # we are in 't', go up a level so we don't create t/t/_tmp
-    chdir '..' or die "Couldn't chdir to ..";
-
-    push @INC, File::Spec->catdir(qw/lib Module Build/, $t_lib);
-
-    # make sure children get @INC pointing to uninstalled files
-    require Cwd;
-    $ENV{PERL5LIB} = File::Spec->catdir(Cwd::cwd(), 'lib');
-  }
+  # We change directories, so expand @INC and $^X to absolute paths
+  # Also add .
+  @INC = (map(File::Spec->rel2abs($_), @INC), ".");
+  $^X = File::Spec->rel2abs($^X);
 }
 
 use Exporter;
@@ -76,34 +66,64 @@ use Config;
 use Cwd ();
 
 # We pass everything through to Test::More
-use vars qw($VERSION @ISA @EXPORT %EXPORT_TAGS $TODO);
+use vars qw($VERSION @ISA @EXPORT $TODO);
+@ISA = ('Exporter');
 $VERSION = 0.01_01;
-@ISA = qw(Test::More); # Test::More isa Exporter
-@EXPORT = @Test::More::EXPORT;
-%EXPORT_TAGS = %Test::More::EXPORT_TAGS;
 
 # We have a few extra exports, but Test::More has a special import()
 # that won't take extra additions.
-my @extra_exports = qw(
-  stdout_of
-  stderr_of
-  stdout_stderr_of
-  slurp
-  find_in_path
-  check_compiler
-  have_module
-  ensure_blib
+@EXPORT = (
+    qw(
+      stdout_of
+      stderr_of
+      stdout_stderr_of
+      slurp
+      find_in_path
+      check_compiler
+      have_module
+      blib_load
+      timed_out
+      $TODO
+    ),
+    @Test::More::EXPORT,
 );
-push @EXPORT, @extra_exports;
-__PACKAGE__->export(scalar caller, @extra_exports);
-# XXX ^-- that should really happen in import()
+
+sub import {
+    my $class = shift;
+    my $caller = caller;
+
+    my @imports;
+
+    while (my $item = shift @_) {
+        if ($item eq 'tests' || $item eq 'skip_all') {
+            my $arg = shift @_;
+            plan($item => $arg);
+        }
+        elsif($item eq 'no_plan') {
+            plan($item);
+        }
+        else {
+            push @imports => $item;
+        }
+    }
+
+    @imports = @EXPORT unless @imports;
+
+    $class->export($caller, @imports);
+}
 
 
 ########################################################################
 
 # always return to the current directory
-{ 
-  my $cwd = Cwd::cwd;
+{
+  my $cwd;
+  # must be done in BEGIN because tmpdir uses it in BEGIN for $ENV{HOME}
+  BEGIN { 
+    $cwd = File::Spec->rel2abs(Cwd::cwd);
+  }
+
+  sub original_cwd { return $cwd }
 
   END {
     # Go back to where you came from!
@@ -114,25 +134,27 @@ __PACKAGE__->export(scalar caller, @extra_exports);
 
 { # backwards compatible temp filename recipe adapted from perlfaq
   my $tmp_count = 0;
-  my $tmp_base_name = sprintf("%d-%d", $$, time());
+  my $tmp_base_name = sprintf("MB-%d-%d", $$, time());
   sub temp_file_name {
     sprintf("%s-%04d", $tmp_base_name, ++$tmp_count)
   }
 }
 ########################################################################
 
-# Setup a temp directory 
-sub tmpdir { 
-  my ($self, $usr_tmp) = @_;
-  return File::Temp::tempdir( 'MB-XXXXXXXX', 
-    CLEANUP => 1, DIR => $ENV{PERL_CORE} ? Cwd::cwd : 
-                         $usr_tmp        ? $usr_tmp : File::Spec->tmpdir 
-  );
+# Setup a temp directory
+sub tmpdir {
+  my ($self, @args) = @_;
+  my $dir = $ENV{PERL_CORE} ? MBTest->original_cwd : File::Spec->tmpdir;
+  return File::Temp::tempdir('MB-XXXXXXXX', CLEANUP => 1, DIR => $dir, @args);
+}
+
+BEGIN {
+  $ENV{HOME} = tmpdir; # don't want .modulebuildrc or other things interfering
 }
 
 sub save_handle {
   my ($handle, $subr) = @_;
-  my $outfile = temp_file_name();
+  my $outfile = File::Spec->catfile(File::Spec->tmpdir, temp_file_name());
 
   local *SAVEOUT;
   open SAVEOUT, ">&" . fileno($handle)
@@ -155,11 +177,11 @@ sub stdout_stderr_of {
   $stdout = stdout_of ( sub {
       $stderr = stderr_of( $subr )
   });
-  return ($stdout, $stderr);
+  return wantarray ? ($stdout, $stderr) : $stdout . $stderr;
 }
 
 sub slurp {
-  my $fh = IO::File->new($_[0]) or die "Can't open $_[0]: $!";
+  open(my $fh, '<', $_[0]) or die "Can't open $_[0]: $!";
   local $/;
   return scalar <$fh>;
 }
@@ -178,29 +200,47 @@ sub exe_exts {
 
 sub find_in_path {
   my $thing = shift;
-  
-  my @path = split $Config{path_sep}, $ENV{PATH};
+
   my @exe_ext = exe_exts();
-  foreach (@path) {
-    my $fullpath = File::Spec->catfile($_, $thing);
+  if ( File::Spec->file_name_is_absolute( $thing ) ) {
     foreach my $ext ( '', @exe_ext ) {
-      return "$fullpath$ext" if -e "$fullpath$ext";
+      return "$thing$ext" if -e "$thing$ext";
+    }
+  }
+  else {
+    my @path = split $Config{path_sep}, $ENV{PATH};
+    foreach (@path) {
+      my $fullpath = File::Spec->catfile($_, $thing);
+      foreach my $ext ( '', @exe_ext ) {
+        return "$fullpath$ext" if -e "$fullpath$ext";
+      }
     }
   }
   return;
 }
 
-# returns ($have_c_compiler, $C_support_feature);
 sub check_compiler {
-  return (1,1) if $ENV{PERL_CORE};
+  if ($ENV{PERL_CORE}) {
+    require IPC::Cmd;
+    if ( $Config{usecrosscompile} && !IPC::Cmd::can_run($Config{cc}) ) {
+      return;
+    }
+    else {
+      return(1,1);
+    }
+  }
 
   local $SIG{__WARN__} = sub {};
 
+  blib_load('Module::Build');
   my $mb = Module::Build->current;
   $mb->verbose( 0 );
 
   my $have_c_compiler;
   stderr_of( sub {$have_c_compiler = $mb->have_c_compiler} );
+  # XXX link_executable() is not yet implemented for Windows
+  # and noexec tmpdir is irrelevant on Windows
+  return ($have_c_compiler, 1) if $^O eq "MSWin32";
 
   # check noexec tmpdir
   my $tmp_exec;
@@ -215,28 +255,60 @@ sub check_compiler {
     );
     $tmp_exec = 0 == system( $exe );
   }
-  return ($have_c_compiler, $mb->feature('C_support'), $tmp_exec);
+  return ($have_c_compiler, $tmp_exec);
 }
 
 sub have_module {
   my $module = shift;
-  return eval "use $module; 1";
+  return eval "require $module; 1";
 }
 
-sub ensure_blib {
-  # Make sure the given module was loaded from blib/, not the larger system
+sub blib_load {
+  # Load the given module and ensure it came from blib/, not the larger system
   my $mod = shift;
+  have_module($mod) or die "Error loading $mod\: $@\n";
+
   (my $path = $mod) =~ s{::}{/}g;
- 
-  local $Test::Builder::Level = $Test::Builder::Level + 1; 
- SKIP: {
-    skip "no blib in core", 1 if $ENV{PERL_CORE};
-    like $INC{"$path.pm"}, qr/\bblib\b/, "Make sure $mod was loaded from blib/"
-      or diag "PERL5LIB: " . ($ENV{PERL5LIB} || '') . "\n" .
-              "PERL5OPT: " . ($ENV{PERL5OPT} || '') . "\n" .
-              "\@INC contains:\n  " . join("\n  ", @INC) . "\n"; 
+  $path .= ".pm";
+  my ($pkg, $file, $line) = caller;
+  unless($ENV{PERL_CORE}) {
+    unless($INC{$path} =~ m/\bblib\b/) {
+      (my $load_from = $INC{$path}) =~ s{$path$}{};
+      die "$mod loaded from '$load_from'\nIt should have been loaded from blib.  \@INC contains:\n  ",
+      join("\n  ", @INC) . "\nFatal error occured in blib_load() at $file, line $line.\n";
+    }
   }
 }
 
+sub timed_out {
+  my ($sub, $timeout) = @_;
+  return unless $sub;
+  $timeout ||= 60;
+
+  my $saw_alarm = 0;
+  eval {
+    local $SIG{ALRM} = sub { $saw_alarm++; die "alarm\n"; }; # NB: \n required
+    alarm $timeout;
+    $sub->();
+    alarm 0;
+  };
+  if ($@) {
+    die unless $@ eq "alarm\n";   # propagate unexpected errors
+  }
+  return $saw_alarm;
+}
+
+sub check_EUI {
+  my $timed_out;
+  stdout_stderr_of( sub {
+      $timed_out = timed_out( sub {
+          ExtUtils::Installed->new(extra_libs => [@INC])
+        }
+      );
+    }
+  );
+  return ! $timed_out;
+}
+
 1;
 # vim:ts=2:sw=2:et:sta
